@@ -1,7 +1,7 @@
 ;; -*- mode: scheme; coding: utf-8 -*-
 
 ;;;;
-;;;; Copyright (C) 2016 - 2017
+;;;; Copyright (C) 2016 - 2018
 ;;;; Free Software Foundation, Inc.
 
 ;;;; This file is part of GNU Guile-CV.
@@ -60,6 +60,10 @@
             im-xor
 	    im-complement
             im-range
+            im-mtimes
+            im-mtimes-channel
+            im-mdivide
+            im-mdivide-channel
 	    im-min
 	    im-max
             im-map
@@ -83,8 +87,8 @@
           im-add-channel
           im-subtract
           im-subtract-channel
-          im-multiply
-          im-multiply-channel
+          im-times
+          im-times-channel
           im-divide
           im-divide-channel)
 
@@ -332,12 +336,21 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
     (f32vector-set! to i (op (f32vector-ref channel i)
                              (f32vector-ref channel-2 i))))))
 
-(define (matrix-multiply c1 width-1 height-1 c2 width-2)
-  (f32vector-matrix-multiply c1 width-1 height-1 c2 width-2))
+(define (mtimes c1 width-1 height-1 c2 width-2)
+  (let ((to (im-make-channel width-2 height-1)))
+    (f32vector-mtimes c1 width-1 height-1 c2 width-2 to)
+    to))
 
-(define (matrix-divide c1 width-1 height-1 c2 width-2)
-  (f32vector-matrix-multiply c1 width-1 height-1
-                             (f32vector-invert c2) width-2))
+(define (mdivide c1 width-1 height-1 c2 width-2)
+  (let* ((height-2 width-1)
+         (i-c2 (im-make-channel width-2 height-2))
+         (n-cell-2 (* width-2 height-2))
+         (to (im-make-channel width-2 height-1)))
+    (f32vector-mtimes c1 width-1 height-1
+                      (f32vector-invert c2 i-c2 #:n-cell n-cell-2)
+                      width-2
+                      to)
+    to))
 
 (define (im-matrix-multdiv-op img-1 img-2 op)
   ;; The product is defined only if the number of columns in img-1 is
@@ -432,56 +445,71 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
     (f32vector-subtract-vectors to n-cell channels)))
 
 
-#;(define-method (im-multiply image (val <number>))
+#;(define-method (im-times image (val <number>))
   (im-map (lambda (p-val) (* p-val val)) image))
 
-#;(define-method (im-multiply-channel channel width height (val <number>))
+#;(define-method (im-times-channel channel width height (val <number>))
   (im-map-channel (lambda (p-val) (* p-val val)) width height channel))
 
-(define-method (im-multiply image (val <number>))
+(define-method (im-times image (val <number>))
   (match image
     ((width height n-chan idata)
      (list width height n-chan
            (let ((map-proc (if (and (> n-chan 1)
                                     (%use-par-map)) par-map map)))
              (map-proc (lambda (channel)
-                         (im-multiply-channel channel width height val))
+                         (im-times-channel channel width height val))
                  idata))))))
 
-(define-method (im-multiply-channel channel width height (val <number>))
+(define-method (im-times-channel channel width height (val <number>))
   (let ((to (im-make-channel width height))
         (n-cell (* width height)))
-    (f32vector-multiply-value channel val to #:n-cell n-cell)
+    (f32vector-times-value channel val to #:n-cell n-cell)
     to))
 
-(define (im-multiply-1 prev images)
+#;(define-method (im-times . images)
+  (apply im-map * images))
+
+#;(define-method (im-times-channel width height . channels)
+  (apply im-map-channel * width height channels))
+
+(define-method (im-times . images)
+  (im-map-la im-times-channel images))
+
+(define-method (im-times-channel width height . channels)
+  (let ((n-cell (* width height))
+        (to (im-make-channel width height)))
+    (f32vector-times-vectors to n-cell channels)))
+
+(define (im-mtimes-1 prev images)
   (if (null? images)
       prev
       (match images
         ((ii . rest)
-         (im-multiply-1 (im-matrix-multdiv-op prev ii matrix-multiply)
-                        rest)))))
+         (im-mtimes-1 (im-matrix-multdiv-op prev ii mtimes)
+                      rest)))))
 
-(define-method (im-multiply . images)
+(define (im-mtimes . images)
   (match images
     ((i1 . rest)
-     (im-multiply-1 i1 rest))
+     (im-mtimes-1 i1 rest))
     (else
      (error "Wrong arguments:" images))))
 
-(define (im-multiply-channel-1 channel width height rest)
+(define (im-mtimes-channel-1 channel width height rest)
   (if (null? rest)
       (values channel width height)
       (match rest
         ((channel-i width-i height-i . rest)
-         (im-multiply-channel-1 (f32vector-matrix-multiply channel width height
-                                                           channel-i width-i)
-                                width-i height rest)))))
+         (let ((to (im-make-channel width-i height)))
+           (im-mtimes-channel-1 (f32vector-mtimes channel width height
+                                                  channel-i width-i to)
+                                width-i height rest))))))
 
-(define-method (im-multiply-channel . rest)
+(define (im-mtimes-channel . rest)
   (match rest
     ((channel width height . rest)
-     (im-multiply-channel-1 channel width height rest))
+     (im-mtimes-channel-1 channel width height rest))
     (else
      (error "Wrong arguments:" rest))))
 
@@ -512,35 +540,54 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
         (f32vector-divide-value channel val to #:n-cell n-cell)
         to)))
 
-(define (im-divide-1 prev images)
+#;(define-method (im-divide . images)
+  (apply im-map / images))
+
+#;(define-method (im-divide-channel width height . channels)
+  (apply im-map-channel / width height channels))
+
+(define-method (im-divide . images)
+  (im-map-la im-divide-channel images))
+
+(define-method (im-divide-channel width height . channels)
+  (let ((n-cell (* width height))
+        (to (im-make-channel width height)))
+    (f32vector-divide-vectors to n-cell channels)))
+
+(define (im-mdivide-1 prev images)
   (if (null? images)
       prev
       (match images
         ((ii . rest)
-         (im-divide-1 (im-matrix-multdiv-op prev ii matrix-divide)
-                        rest)))))
+         (im-mdivide-1 (im-matrix-multdiv-op prev ii mdivide)
+                       rest)))))
 
-(define-method (im-divide . images)
+(define (im-mdivide . images)
   (match images
     ((i1 . rest)
-     (im-divide-1 i1 rest))
+     (im-mdivide-1 i1 rest))
     (else
      (error "Wrong arguments:" images))))
 
-(define (im-divide-channel-1 channel width height rest)
+(define (im-mdivide-channel-1 channel width height rest)
   (if (null? rest)
       (values channel width height)
       (match rest
         ((channel-i width-i height-i . rest)
-         (im-divide-channel-1 (f32vector-matrix-multiply channel width height
-                                                         (f32vector-invert channel-i)
-                                                         width-i)
-                                width-i height rest)))))
+         (let ((i-ci (im-make-channel width-i height-i))
+               (n-cell-i (* width-i height-i))
+               (to (im-make-channel width-i height)))
+           (im-mdivide-channel-1 (f32vector-mtimes channel width height
+                                                   (f32vector-invert channel-i i-ci
+                                                                     #:n-cell n-cell-i)
+                                                   width-i
+                                                   to)
+                                 width-i height rest))))))
 
-(define-method (im-divide-channel . rest)
+(define (im-mdivide-channel . rest)
   (match rest
     ((channel width height . rest)
-     (im-divide-channel-1 channel width height rest))
+     (im-mdivide-channel-1 channel width height rest))
     (else
      (error "Wrong arguments:" rest))))
 
@@ -919,7 +966,10 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                        idata))))))
 
 (define (im-invert-channel channel width height)
-  (f32vector-invert channel))
+  (let ((to (im-make-channel width height))
+        (n-cell (* width height)))
+    (f32vector-invert channel to #:n-cell n-cell)
+    to))
 
 (define* (im-normalize image #:key (val 255.0))
   (match image
