@@ -58,7 +58,11 @@
 	    im-unpadd
 	    im-unpadd-channel
             im-clip
-            im-clip-channel))
+            im-clip-channel
+            im-local-minima
+            im-local-minima-channel
+            im-local-maxima
+            im-local-maxima-channel))
 
 
 #;(g-export )
@@ -283,6 +287,81 @@
       (else
         (error "Clip failed.")))))
 
+(define* (im-local-minima image
+                          #:key (con 8)
+                          (marker 1.0)
+                          (threshold +float-max+)
+                          (borders? #f)
+                          (plateaus? #f)
+                          (epsilon 1.0e-4))
+  (match image
+    ((width height n-chan idata)
+     (list width height n-chan
+           (let ((map-proc (if (and (> n-chan 1)
+                                    (%use-par-map)) par-map map)))
+	     (map-proc (lambda (channel)
+			 (im-local-minima-channel channel width height
+                                                  #:con con
+                                                  #:marker marker
+                                                  #:threshold threshold
+                                                  #:borders? borders?
+                                                  #:plateaus? plateaus?
+                                                  #:epsilon epsilon))
+	       idata))))))
+
+(define* (im-local-minima-channel channel width height
+                                  #:key (con 8)
+                                  (marker 1.0)
+                                  (threshold +float-max+)
+                                  (borders? #f)
+                                  (plateaus? #f)
+                                  (epsilon 1.0e-4))
+  (let ((to (im-make-channel width height))
+        (n-cell (* width height)))
+    (case (vigra-local-minima channel to width height con marker threshold
+                              borders? plateaus? epsilon)
+      ((0) to)
+      (else
+        (error "Local minima failed.")))))
+
+(define* (im-local-maxima image
+                          #:key (con 8)
+                          (marker 1.0)
+                          (threshold (- +float-max+))
+                          (borders? #f)
+                          (plateaus? #f)
+                          (epsilon 1.0e-4))
+  (match image
+    ((width height n-chan idata)
+     (list width height n-chan
+           (let ((map-proc (if (and (> n-chan 1)
+                                    (%use-par-map)) par-map map)))
+	     (map-proc (lambda (channel)
+			 (im-local-maxima-channel channel width height
+                                                  #:con con
+                                                  #:marker marker
+                                                  #:threshold threshold
+                                                  #:borders? borders?
+                                                  #:plateaus? plateaus?
+                                                  #:epsilon epsilon))
+
+	       idata))))))
+
+(define* (im-local-maxima-channel channel width height
+                                  #:key (con 8)
+                                  (marker 1.0)
+                                  (threshold (- +float-max+))
+                                  (borders? #f)
+                                  (plateaus? #f)
+                                  (epsilon 1.0e-4))
+  (let ((to (im-make-channel width height))
+        (n-cell (* width height)))
+    (case (vigra-local-maxima channel to width height con marker threshold
+                              borders? plateaus? epsilon)
+      ((0) to)
+      (else
+        (error "Local maxima failed.")))))
+
 
 ;;;
 ;;; Guile vigra low level API
@@ -340,6 +419,40 @@
 			lower
 			upper))
 
+(define (vigra-local-minima from to width height con marker threshold
+                            borders? plateaus? epsilon)
+  (vigra-local-minima-c (bytevector->pointer from)
+                        (bytevector->pointer to)
+                        width
+                        height
+                        (case con
+                          ((8) 1)
+                          ((4) 0)
+                          (else
+                           (error "No such connectivity: " con)))
+                        marker
+                        threshold
+                        (if borders? 1 0)
+                        (if plateaus? 1 0)
+                        epsilon))
+
+(define (vigra-local-maxima from to width height con marker threshold
+                            borders? plateaus? epsilon)
+  (vigra-local-maxima-c (bytevector->pointer from)
+                        (bytevector->pointer to)
+                        width
+                        height
+                        (case con
+                          ((8) 1)
+                          ((4) 0)
+                          (else
+                           (error "No such connectivity: " con)))
+                        marker
+                        threshold
+                        (if borders? 1 0)
+                        (if plateaus? 1 0)
+                        epsilon))
+
 
 ;;;
 ;;; Vigra_c bindings
@@ -414,3 +527,33 @@
 			    int		;; from height
 			    float	;; lower
 			    float)))	;; upper
+
+(define vigra-local-minima-c
+    (pointer->procedure int
+		      (dynamic-func "vigra_localminima_c"
+				    %libvigra-c)
+		      (list '*		;; from channel
+			    '*		;; to channel
+			    int		;; from width
+			    int		;; from height
+                            int		;; 8-con?
+                            float	;; marker
+                            float	;; threshold
+                            int		;; borders?
+                            int		;; plateaus?
+                            float)))	;; epsilon
+
+(define vigra-local-maxima-c
+    (pointer->procedure int
+		      (dynamic-func "vigra_localmaxima_c"
+				    %libvigra-c)
+		      (list '*		;; from channel
+			    '*		;; to channel
+			    int		;; from width
+			    int		;; from height
+                            int		;; 8-con?
+                            float	;; marker
+                            float	;; threshold
+                            int		;; borders?
+                            int		;; plateaus?
+                            float)))	;; epsilon
