@@ -205,12 +205,12 @@
                                                #:radius radius))
 		       idata))))))
 
-(define* (im-delineate-channel channel width height
+#;(define* (im-delineate-channel channel width height
                                #:key (threshold  10) (radius 2))
-  (let* ((channel-min (im-disc-erode-channel channel width height radius))
-         (channel-max (im-disc-dilate-channel channel width height radius))
-         (to (im-make-channel width height))
-         (n-cell (* width height)))
+  (let ((channel-min (im-disc-erode-channel channel width height radius))
+        (channel-max (im-disc-dilate-channel channel width height radius))
+        (to (im-make-channel width height))
+        (n-cell (* width height)))
     (do ((i 0
 	    (+ i 1)))
 	((= i n-cell) to)
@@ -228,6 +228,17 @@
                                 mini
                                 maxi)))))))
 
+(define* (im-delineate-channel channel width height
+                               #:key (threshold  10) (radius 2))
+  (let ((to (im-make-channel width height)))
+    (f32vector-delineate channel
+                         (im-disc-erode-channel channel width height radius)
+                         (im-disc-dilate-channel channel width height radius)
+                         to
+                         #:n-cell (* width height)
+                         #:threshold threshold)
+    to))
+
 (define* (im-distance-map image
                           #:key (bg 'black) (mode 'euclidian))
   (match image
