@@ -33,13 +33,17 @@
   #:export (;; misc.
             pointer-address-size-c
 
-            ;; floats
-            float-to-int-c
-            float-equal-c
+            ;; limits
+            float-min-c
+            float-max-c
 
             ;; bounding box
             bb-intersect-c
 
+            ;; floats
+            float-to-int-c
+            float-equal-c
+
             ;; f32vectors
             f32vector-min-c
             f32vector-max-c
@@ -67,6 +71,8 @@
             f32vector-is-a-seed-c
             f32vector-scale-c
             f32vector-to-s32vector-c
+            f32vector-delineate-c
+
             ;; glcm
             glcm-c))
 
@@ -82,6 +88,34 @@
                       (list)))
 
 
+;;;
+;;; limits
+;;;
+
+(define float-min-c
+  (pointer->procedure float
+                      (dynamic-func "float_min_c"
+                                    %libguile-cv)
+                      (list)))
+
+(define float-max-c
+  (pointer->procedure float
+                      (dynamic-func "float_max_c"
+                                    %libguile-cv)
+                      (list)))
+
+
+;;;
+;;; bouding box
+;;;
+
+(define bb-intersect-c
+  (pointer->procedure int
+                      (dynamic-func "bb_intersect_c"
+                                    %libguile-cv)
+                      (list int int int int int int int int)))
+
+
 ;;;
 ;;; floats
 ;;;
@@ -101,17 +135,6 @@
                             float)))	;; precision
 
 
-;;;
-;;; bouding box
-;;;
-
-(define bb-intersect-c
-  (pointer->procedure int
-                      (dynamic-func "bb_intersect_c"
-                                    %libguile-cv)
-                      (list int int int int int int int int)))
-
-
 ;;;
 ;;; f32vectors
 ;;;
@@ -349,6 +372,17 @@
                             int   ;; n-cell
                             '*))) ;; to
 
+(define f32vector-delineate-c
+  (pointer->procedure int
+                      (dynamic-func "f32vector_delineate_c"
+                                    %libguile-cv)
+                      (list '*		;; chan
+                            '*		;; chan-min
+                            '*		;; chan-max
+                            int		;; n-cell
+                            int		;; threshold
+                            '*)))	;; to
+
 
 ;;;
 ;;; glcm
