@@ -27,6 +27,7 @@
 #include <stdlib.h>
 #include <stddef.h>
 #include <limits.h>
+#include <float.h>
 #include <math.h>
 /* #include <libguile.h> */
 
@@ -51,6 +52,22 @@ int im_fast_channel_offset (int i,
 }
 
 
+/*
+ * limits
+ *
+*/
+
+float float_min_c ()
+{
+  return (FLT_MIN);
+}
+
+float float_max_c ()
+{
+  return (FLT_MAX);
+}
+
+
 /*
  * bounding box
  *
@@ -639,6 +656,36 @@ int f32vector_to_s32vector_c (float *v,
   return 0;
 }
 
+int f32vector_delineate_c (float *v,
+                           float *v_min,
+                           float *v_max,
+                           int n_cell,
+                           int threshold,
+                           float *to)
+{
+  int i;
+  float ori, mini, maxi, diff;
+
+  for (i = 0; i < n_cell; i++) {
+    ori = v[i];
+    mini = v_min[i];
+    maxi = v_max[i];
+    diff = maxi - mini;
+    if (diff < threshold) {
+      to[i] = ori;
+    }
+    else {
+      if ((ori - mini) < (maxi - ori)) {
+        to[i] = mini;
+      }
+      else {
+        to[i] = maxi;
+      }
+    }
+  }
+  return 0;
+}
+
 
 /*
  * glcm
